<?php defined('SYSPATH') OR die('No direct access allowed.');

/**
 * Controller for SysNagios GUI extension
 *
 **/
class SysNagios_Controller extends Authenticated_Controller
{

	const NEW_HOST = 1;
	const EXISTING_HOST = 2;

	private static $getconfigs_path = "/usr/local/sysnagios_server/getconfigs.sh";

	/**
	 * Head "stuff"™
	 *
	 * @return void
	 **/
	protected function _initHead()
	{
		$this->template->css[] = 'modules/sysnagios/views/sysnagios/css/sysnagios.css';
		$this->template->js_header = $this->add_view('js_header');
		$sysnagios_js_path = 'modules/sysnagios/views/sysnagios/js/';
		$this->template->js[] = $sysnagios_js_path.'sysnagios.js';
	}

	/**
	 * Page when generating config for a new host
	 *
	 * @return void
	 **/
	public function new_host()
	{
		$this->template->content = $this->add_view('sysnagios/index');
		$this->template->content->mode = self::NEW_HOST;
		$this->_initHead();
		$this->template->title = _("Generate config for a new host");
		$this->template->disable_refresh = true;

		$mon_node_show = SysNagios_Model::mon_node_show();
		if ($mon_node_show[0] !== "No nodes configured") {
			$nodes = SysNagios_Model::parse_mon_node_show($mon_node_show);
			// Separate nodes by type in optgroups
			$node_names = array("Self" => array("--self" => "--self"),"Master" => array(), "Peer" => array(), "Poller" => array());
			foreach ($nodes as $node) {
				switch ($node['type']) {
					case 'master':
						$node_names['Master'][$node['name']] = $node['name'];
						break;
					case 'peer':
						$node_names['Peer'][$node['name']] = $node['name'];
						break;
					case 'poller':
						$node_names['Poller'][$node['name']] = $node['name'];
						break;
					default:
						throw new Exception("Invalid node type");
				}
			}
			foreach ($node_names as $node_name => $array) {
				if (count($node_names[$node_name]) === 0) {
					unset($node_names[$node_name]);
				}
			}
			$this->template->content->nodes = $node_names;
		} else {
			$this->template->content->nodes = array("Self" => array("--self" => "--self"));
		}
	}

	/**
	 * Page when generating config for an existing host
	 *
	 * @return void
	 **/
	public function existing_host($host)
	{
		$this->template->content = $this->add_view('sysnagios/index');
		$this->template->content->mode = self::EXISTING_HOST;
		$this->_initHead();
		$this->template->title = _("Generate config for an existing host");
		$this->template->disable_refresh = true;

		$this->template->content->host = $host;
		try {
			$node_list = SysNagios_Model::get_node_list();
			$hostgroups = SysNagios_Model::get_hostgroup_from_member($host);
			$node = SysNagios_Model::match_node($host, $hostgroups, $node_list);
		} catch(Exception $e) {
			$node = "--self";
		}
		$this->template->content->nodes = array($node => $node);
	}

	/**
	 * Translated helptexts for this controller
	 *
	 * @return void
	 **/
	public static function _helptexts($id)
	{
		$helptexts = array(
			'host' => _('Enter either ip, hostname or FQDN for the host you which to generate config for. Separate addresses with a space character if you want to enter more than one host.'),
			'node' => _('Choose which node the generate config script should be executed on. Choosing self as node will execute script on the current machine.')
		);
		if (array_key_exists($id, $helptexts)) {
			echo $helptexts[$id];
		}
		else
			echo sprintf(_("This helptext ('%s') is not translated yet"), $id);
	}

	/**
	 * Generate config for a host
	 *
	 * @param $hosts string
	 * @param $node string
	 * @return string
	 **/
	public function submit_host()
	{
		if ($_POST) {
			$hosts = $_POST['hosts'];
			$node = $_POST['node'];
		} else {
			return print("No post data set");
		}
		// Hosts are submitted separated by spaces
		$hosts = explode(" ", $hosts);
		$parsed_hosts = array();
		foreach ($hosts as $host) {
			try {
				$host = SysNagios_Model::parse_host_address($host);
			} catch(Exception $e) {
				return print($e->getMessage());
			}
			$parsed_hosts[] = $host['ip'];
		}
		if (SysNagios_Model::lock_file_exists($node)) {
			return print("Lock is in effect. Another operation is executing, try again in a moment");
		}
		$lock = SysNagios_Model::create_lock_file($node);
		if ($lock === true) {
			$getconfigs_exists = SysNagios_Model::getconfigs_exist(self::$getconfigs_path, $node);
			if ($getconfigs_exists !== true) {
				SysNagios_Model::remove_lock_file($node);
				return print("Error: " . self::$getconfigs_path . " doesn't exist on " . $node);
			}
			$cmd = "sh " . self::$getconfigs_path . " " . implode(" ", $parsed_hosts);
			try {
				$mon_node_ctrl_output = SysNagios_Model::mon_node_ctrl($node, $cmd);
			} catch(Exception $e) {
				return print($e->getMessage());
			}
			$mon_node_ctrl_output = implode("\n", $mon_node_ctrl_output);
		} else {
			return print($lock);
		}
		$unlock = SysNagios_Model::remove_lock_file($node);
		if ($unlock === true) {
			return print($mon_node_ctrl_output);
		}
		return print($mon_node_ctrl_output . "Execution was successful but lock removal failed. \nTry to force lock removal with the button below.");
	}

	/**
	 * Forces removal of lock-file
	 *
	 * @param $node string
	 * @return string
	 **/
	public function force_unlock($node)
	{
		if (SysNagios_Model::lock_file_exists($node)) {
			$lock_removed = SysNagios_Model::remove_lock_file($node);
			if ($lock_removed === true) {
				return print("Lock was successfully removed from node: " . $node);
			}
			return print("Error: Lock was not removed from node: " . $lock_removed);
		}
		return print("No lock file exists on the node: " . $node);
	}

} // END class
